import copy


class Position:
    __index: int = 0
    __line: int = 0
    __column: int = 0

    def __init__(self, index: int = 0, line: int = 0, column: int = 0) -> None:
        self.__index = index
        self.__line = line
        self.__column = column

    def advance(self, chr: str = "\n") -> None:
        if len(chr) != 1:
            raise TypeError(
                f"advance() expected character, but string of length {len(chr)} found"
            )

        if chr == "\n":
            self.__line += 1
            self.__column = 0

        else:
            self.__column += 1

        self.__index += 1

    @property
    def index(self) -> int:
        return copy.copy(self.__index)

    @property
    def line(self) -> int:
        return copy.copy(self.__line)

    @property
    def column(self) -> int:
        return copy.copy(self.__column)

    def __copy__(self) -> "Position":
        return Position(
            copy.copy(self.__index), copy.copy(
                self.__line), copy.copy(self.__column)
        )

    def __deepcopy__(self, memo: dict) -> "Position":
        return Position(
            copy.deepcopy(self.__index, memo),
            copy.deepcopy(self.__line, memo),
            copy.deepcopy(self.__column, memo),
        )

    def __repr__(self) -> str:
        return f"Position({self.__index}, {self.__line}, {self.__column})"

    def __str__(self) -> str:
        return f"{self.__line + 1}:{self.__column + 1}"

    def __int__(self) -> int:
        return copy.copy(self.__index)
